# -*- coding: utf-8 -*-
from django.views.generic import ListView, DetailView, FormView, TemplateView
from django.urls import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.utils.functional import cached_property
from django.views.generic.base import TemplateResponseMixin
from django.utils import translation

from .settings import PAGINATE_BY
from .models import Newsletter, NewsletterList
from .forms import SubscriptionForm, UnsubscribeForm
from .utils import ajaxify_template_var


class AJAXResponseMixin(TemplateResponseMixin):
    ajax_template_name = None

    def get_template_names(self):
        names = super(AJAXResponseMixin, self).get_template_names()

        if self.request.is_ajax():
            if self.ajax_template_name:
                names = [self.ajax_template_name] + names
            else:
                names = ajaxify_template_var(names) + names
        return names


class NewsletterListView(AJAXResponseMixin, ListView):
    model = Newsletter
    context_object_name = "newsletters"
    template_name = "courriers/newsletter_list.html"
    paginate_by = PAGINATE_BY

    def dispatch(self, *args, **kwargs):
        return super(NewsletterListView, self).dispatch(*args, **kwargs)

    @cached_property
    def newsletter_list(self):
        return get_object_or_404(
            NewsletterList.objects.all(), slug=self.kwargs.get("slug")
        )

    def get_queryset(self):
        lang = translation.get_language()
        qs = self.newsletter_list.newsletters.status_online()
        if lang:
            qs = qs.filter(newsletter_segment__lang=lang)
        qs = qs.order_by("-published_at")
        return qs

    def get_context_data(self, **kwargs):
        context = super(NewsletterListView, self).get_context_data(**kwargs)
        context["newsletter_list"] = self.newsletter_list
        return context


class NewsletterDetailView(AJAXResponseMixin, DetailView):
    model = Newsletter
    context_object_name = "newsletter"
    template_name = "courriers/newsletter_detail.html"

    def get_queryset(self):
        return self.model.objects.status_online()

    def get_context_data(self, **kwargs):
        context = super(NewsletterDetailView, self).get_context_data(**kwargs)

        context["newsletter_list"] = self.object.newsletter_list

        return context


class BaseNewsletterListFormView(AJAXResponseMixin, FormView):
    model = NewsletterList
    context_object_name = "newsletter_list"

    @cached_property
    def object(self):
        slug = self.kwargs.get("slug", None)

        if slug:
            return get_object_or_404(self.model, slug=slug)

        return None

    def post(self, request, *args, **kwargs):
        self.get_context_data(**kwargs)

        form_class = self.get_form_class()
        form = self.get_form(form_class)

        if form.is_valid():
            return self.form_valid(form)

        return self.form_invalid(form)

    def get_form_kwargs(self):
        kwargs = super(BaseNewsletterListFormView, self).get_form_kwargs()

        if self.object:
            kwargs["newsletter_list"] = self.object

        return kwargs

    def get_context_data(self, **kwargs):
        context = super(BaseNewsletterListFormView, self).get_context_data(**kwargs)

        if self.object:
            context[self.context_object_name] = self.object

        return context

    def form_valid(self, form):
        if self.request.user.is_authenticated:
            form.save(self.request.user)
        else:
            form.save()

        return HttpResponseRedirect(self.get_success_url())


class NewsletterListSubscribeView(BaseNewsletterListFormView):
    template_name = "courriers/newsletter_list_subscribe_form.html"
    form_class = SubscriptionForm

    def get_success_url(self):
        return reverse("newsletter_list_subscribe_done")


class NewsletterRawDetailView(AJAXResponseMixin, DetailView):
    model = Newsletter
    template_name = "courriers/newsletter_raw_detail.html"

    def get_context_data(self, **kwargs):
        context = super(NewsletterRawDetailView, self).get_context_data(**kwargs)

        context["items"] = self.object.items.all()

        for item in context["items"]:
            item.newsletter = self.object

        return context


class NewsletterListUnsubscribeView(BaseNewsletterListFormView):
    template_name = "courriers/newsletter_list_unsubscribe.html"

    def get_form_class(self):
        return UnsubscribeForm

    def get_initial(self):
        initial = super(NewsletterListUnsubscribeView, self).get_initial()
        email = self.request.GET.get("email", None)

        if email:
            initial["email"] = email

        return initial.copy()

    def get_success_url(self):
        if self.object:
            return reverse(
                "newsletter_list_unsubscribe_done", kwargs={"slug": self.object.slug}
            )

        return reverse("newsletter_list_unsubscribe_done")


class NewsletterListUnsubscribeDoneView(AJAXResponseMixin, TemplateView):
    template_name = "courriers/newsletter_list_unsubscribe_done.html"
    model = NewsletterList
    context_object_name = "newsletter_list"

    def get_context_data(self, **kwargs):
        context = super(NewsletterListUnsubscribeDoneView, self).get_context_data(
            **kwargs
        )

        slug = self.kwargs.get("slug", None)

        if slug:
            context[self.context_object_name] = get_object_or_404(self.model, slug=slug)

        return context


class NewsletterListSubscribeDoneView(AJAXResponseMixin, TemplateView):
    template_name = "courriers/newsletter_list_subscribe_done.html"
    model = NewsletterList
    context_object_name = "newsletter_list"



DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": "",
        "USER": "",
        "PASSWORD": "",
        "HOST": "",
        "PORT": "",
    }
}

SITE_ID = 1
DEBUG = True

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "courriers",
    "courriers.tests",
]

AUTH_USER_MODEL = "tests.User"

SECRET_KEY = "blabla"

ROOT_URLCONF = "courriers.urls"

try:
    from .temp import *  # noqa
except ImportError:
    pass


EMAIL_BACKEND = "django.core.mail.backends.console.EmailBackend"


LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "verbose": {
            "format": "%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s"
        }
    },
    "handlers": {
        "stream": {
            "level": "DEBUG",
            "class": "logging.StreamHandler",
            "formatter": "verbose",
        }
    },
    "loggers": {
        "django.request": {"handlers": ["stream"], "level": "DEBUG", "propagate": True},
        "courriers": {"handlers": ["stream"], "level": "DEBUG", "propagate": True},
    },
}

CELERY_ALWAYS_EAGER = True
CELERY_EAGER_PROPAGATES_EXCEPTIONS = True

MIDDLEWARE = [
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]



from __future__ import absolute_import

from celery.task import task


@task(bind=True)
def subscribe(self, email, newsletter_list_id, user_id=None, **kwargs):
    from courriers.backends import get_backend
    from courriers.models import NewsletterList
    from courriers import signals

    from django.contrib.auth import get_user_model

    User = get_user_model()

    backend = get_backend()()

    newsletter_list = None

    if newsletter_list_id:
        newsletter_list = NewsletterList.objects.get(pk=newsletter_list_id)

    user = None

    if user_id is not None:
        user = User.objects.get(pk=user_id)
    else:
        user = User.objects.filter(email=email).last()

    if user:
        signals.subscribed.send(sender=User, user=user, newsletter_list=newsletter_list)

    else:
        try:
            backend.subscribe(newsletter_list.list_id, email)
        except Exception as e:
            raise self.retry(exc=e, countdown=60)


@task(bind=True)
def unsubscribe(self, email, newsletter_list_id=None, user_id=None, **kwargs):
    from courriers.backends import get_backend
    from courriers.models import NewsletterList
    from courriers import signals

    from django.contrib.auth import get_user_model

    User = get_user_model()

    newsletter_lists = NewsletterList.objects.all()

    if newsletter_list_id:
        newsletter_lists = NewsletterList.objects.filter(pk=newsletter_list_id)

    user = None

    if user_id is not None:
        user = User.objects.get(pk=user_id)
    else:
        user = User.objects.filter(email=email).last()

    if user:
        for newsletter_list in newsletter_lists:
            signals.unsubscribed.send(
                sender=User, user=user, newsletter_list=newsletter_list
            )
    else:
        backend = get_backend()()

        for newsletter in newsletter_lists:
            backend.unsubscribe(newsletter.list_id, email)



from django.contrib import admin
from django.conf.urls import url
from django import forms
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext as _
from django.http import HttpResponseRedirect
from django.urls import reverse

from .models import Newsletter, NewsletterItem, NewsletterList, NewsletterSegment


class NewsletterItemInline(admin.TabularInline):
    model = NewsletterItem


class NewsletterAdminForm(forms.ModelForm):
    def clean(self, *args, **kwargs):
        segment = self.cleaned_data.get("newsletter_segment")
        newsletter_list = self.cleaned_data.get("newsletter_list")

        if (
            segment
            and newsletter_list
            and segment.newsletter_list_id != newsletter_list.pk
        ):
            self.add_error(
                "newsletter_segment",
                "The segment is not attached to this newsletter list",
            )

    class Meta:
        model = Newsletter
        exclude = ()


class NewsletterAdmin(admin.ModelAdmin):
    change_form_template = "admin/courriers/newsletter/change_form.html"

    list_display = (
        "name",
        "headline",
        "published_at",
        "status",
        "newsletter_list_link",
    )
    list_filter = ("published_at", "status")
    inlines = [NewsletterItemInline]
    form = NewsletterAdminForm

    def get_queryset(self, request):
        return (
            super(NewsletterAdmin, self)
            .get_queryset(request)
            .select_related("newsletter_list")
        )

    def get_urls(self):
        urls = super(NewsletterAdmin, self).get_urls()
        my_urls = [
            url(
                r"^send/(?P<newsletter_id>(\d+))/$",
                self.send_newsletter,
                name="send_newsletter",
            )
        ]
        return my_urls + urls

    def send_newsletter(self, request, newsletter_id):
        from courriers.backends import get_backend

        backend_klass = get_backend()
        backend = backend_klass()

        newsletter = get_object_or_404(Newsletter, pk=newsletter_id)
        backend.send_mails(newsletter)

        self.message_user(request, _('The newsletter "%s" has been sent.') % newsletter)
        return HttpResponseRedirect(
            reverse("admin:courriers_newsletter_change", args=(newsletter.id,))
        )

    def newsletter_list_link(self, obj):
        url = reverse(
            "admin:courriers_newsletterlist_change", args=(obj.newsletter_list.id,)
        )
        return '<a href="%(url)s">%(name)s</a>' % {
            "url": url,
            "name": obj.newsletter_list.name,
        }

    newsletter_list_link.allow_tags = True


class NewsletterListAdmin(admin.ModelAdmin):
    list_display = ("name", "slug", "created_at")


class NewsletterSegmentAdmin(admin.ModelAdmin):
    list_display = ("name", "newsletter_list", "lang")


admin.site.register(Newsletter, NewsletterAdmin)
admin.site.register(NewsletterList, NewsletterListAdmin)
admin.site.register(NewsletterSegment, NewsletterSegmentAdmin)



from django import forms
from django.utils.translation import ugettext_lazy as _

from .backends import get_backend
from .tasks import subscribe, unsubscribe


class SubscriptionForm(forms.Form):
    receiver = forms.EmailField(
        max_length=250,
        required=True,
        widget=forms.TextInput(attrs={"placeholder": _(u"Your email"), "size": "30"}),
    )

    def __init__(self, *args, **kwargs):
        self.newsletter_list = kwargs.pop("newsletter_list", None)

        backend_klass = get_backend()

        self.backend = backend_klass()

        super(SubscriptionForm, self).__init__(*args, **kwargs)

    def save(self, user=None):
        subscribe.delay(
            self.cleaned_data.get("receiver"),
            self.newsletter_list.pk,
            user_id=user.pk if user else None,
        )


class UnsubscribeForm(forms.Form):
    email = forms.EmailField(
        max_length=250,
        required=True,
        widget=forms.TextInput(attrs={"placeholder": _(u"Your email"), "size": "30"}),
    )
    from_all = forms.BooleanField(required=False)

    def __init__(self, *args, **kwargs):
        self.newsletter_list = kwargs.pop("newsletter_list", None)

        backend_klass = get_backend()

        self.backend = backend_klass()

        super(UnsubscribeForm, self).__init__(*args, **kwargs)

    def save(self, user=None):
        from_all = self.cleaned_data.get("from_all", False)

        if from_all or not self.newsletter_list:
            unsubscribe.delay(
                self.cleaned_data.get("email"), user_id=getattr(user, "pk", None)
            )
        else:
            unsubscribe.delay(
                self.cleaned_data["email"],
                newsletter_list_id=self.newsletter_list.pk,
                user_id=getattr(user, "pk", None),
            )



# -*- coding: utf-8 -*-
import mock

from django.test import TestCase
from django.contrib.auth import get_user_model
from django.urls import reverse
from django.utils import timezone as datetime
from django.core import mail

from courriers.forms import SubscriptionForm, UnsubscribeForm
from courriers.models import Newsletter, NewsletterList, NewsletterSegment
from courriers import settings
from courriers.tasks import subscribe, unsubscribe

from .models import NewsletterSubscriber

User = get_user_model()


class BaseBackendTests(TestCase):
    def setUp(self):
        from courriers.backends import get_backend

        self.backend_klass = get_backend()
        self.backend = self.backend_klass()

        current = datetime.now().strftime("%Y-%m-%d %H:%M")

        self.monthly = NewsletterList.objects.create(
            name="TestMonthly", slug="testmonthly", list_id=1
        )
        self.weekly = NewsletterList.objects.create(
            name="TestWeekly", slug="testweekly", list_id=2
        )

        self.segment_monthly = NewsletterSegment.objects.create(
            name="monthly", segment_id=3, newsletter_list=self.monthly
        )
        self.segment_weekly = NewsletterSegment.objects.create(
            name="weekly", segment_id=3, newsletter_list=self.weekly
        )

        self.segment_monthly_fr = NewsletterSegment.objects.create(
            name="monthly fr", segment_id=3, newsletter_list=self.monthly, lang="fr"
        )
        self.segment_weekly_fr = NewsletterSegment.objects.create(
            name="weekly fr", segment_id=4, newsletter_list=self.weekly, lang="fr"
        )

        self.segment_monthly_en = NewsletterSegment.objects.create(
            name="monthly en", segment_id=4, newsletter_list=self.monthly, lang="en-us"
        )

        self.nl_monthly = Newsletter.objects.create(
            name="3000 projets financés %s" % current,
            published_at=datetime.now() - datetime.timedelta(hours=2),
            status=Newsletter.STATUS_ONLINE,
            newsletter_list=self.monthly,
            newsletter_segment=self.segment_monthly,
        )

        self.nl_monthly_fr = Newsletter.objects.create(
            name="3000 projets financés %s [monthly][fr]" % current,
            published_at=datetime.now() - datetime.timedelta(hours=2),
            status=Newsletter.STATUS_ONLINE,
            newsletter_list=self.monthly,
            newsletter_segment=self.segment_monthly_fr,
        )
        self.nl_monthly_en = Newsletter.objects.create(
            name="3000 projects %s" % current,
            published_at=datetime.now() - datetime.timedelta(hours=2),
            status=Newsletter.STATUS_ONLINE,
            newsletter_list=self.monthly,
            newsletter_segment=self.segment_monthly_en,
        )

        self.nl_weekly = Newsletter.objects.create(
            name="3000 projets financés %s [monthly][en-us]" % current,
            published_at=datetime.now() - datetime.timedelta(hours=2),
            status=Newsletter.STATUS_ONLINE,
            newsletter_list=self.weekly,
            newsletter_segment=self.segment_weekly,
        )

        self.nl_weekly_fr = Newsletter.objects.create(
            name="3000 projets financés %s [weekly][fr]" % current,
            published_at=datetime.now() - datetime.timedelta(hours=2),
            status=Newsletter.STATUS_ONLINE,
            newsletter_list=self.weekly,
            newsletter_segment=self.segment_weekly_fr,
        )

        self.user = User.objects.create_user("adele", "adele@ulule.com", "$ecret")

    def test_registration(self):
        self.backend.register("adele@ulule.com", self.monthly, "fr")
        self.backend.register("adele@ulule.com", self.weekly, "fr")

        subscriber = NewsletterSubscriber.objects.filter(
            email="adele@ulule.com", newsletter_list=self.monthly, is_unsubscribed=False
        )
        self.assertEqual(subscriber.count(), 1)

        subscriber2 = NewsletterSubscriber.objects.filter(
            email="adele@ulule.com", newsletter_list=self.weekly, is_unsubscribed=False
        )
        self.assertEqual(subscriber2.count(), 1)

        # Unsubscribe from all
        self.backend.unregister("adele@ulule.com")

        subscriber = NewsletterSubscriber.objects.filter(
            email="adele@ulule.com", is_unsubscribed=True
        )
        self.assertEqual(subscriber.count(), 2)  # subscriber is unsubscribed from all

        # Subscribe to all
        self.backend.register("adele@ulule.com", self.monthly, "fr")
        self.backend.register("adele@ulule.com", self.weekly, "fr")

        subscriber = NewsletterSubscriber.objects.get(
            email="adele@ulule.com", newsletter_list=self.monthly, is_unsubscribed=False
        )

        # Unsubscribe from monthly
        self.backend.unregister(subscriber.email, self.monthly)

        unsubscriber = NewsletterSubscriber.objects.filter(
            email="adele@ulule.com", newsletter_list=self.monthly, is_unsubscribed=True
        )
        self.assertEqual(unsubscriber.count(), 1)

        unsubscriber = NewsletterSubscriber.objects.filter(
            email="adele@ulule.com", newsletter_list=self.weekly, is_unsubscribed=False
        )
        self.assertEqual(unsubscriber.count(), 1)


class SimpleBackendTests(BaseBackendTests):
    def test_registration(self):
        super(SimpleBackendTests, self).test_registration()

        self.backend.register("adele@ulule.com", self.nl_monthly.newsletter_list)

        self.backend.send_mails(
            self.nl_monthly,
            subscribers=NewsletterSubscriber.objects.filter(
                newsletter_list=self.monthly
            ),
        )

        self.assertEqual(len(mail.outbox), 1)
        out = len(mail.outbox)

        self.backend.register(
            "adele@ulule.com", self.nl_monthly_fr.newsletter_list, "fr"
        )

        self.backend.send_mails(
            self.nl_monthly_fr,
            subscribers=NewsletterSubscriber.objects.filter(
                newsletter_list=self.monthly
            ),
        )

        self.assertEqual(len(mail.outbox) - out, 1)
        out = len(mail.outbox)

        self.backend.register("adele@ulule.com", self.monthly, "en-us")

        self.backend.send_mails(
            self.nl_monthly_en,
            subscribers=NewsletterSubscriber.objects.filter(
                newsletter_list=self.monthly
            ),
        )

        self.assertEqual(len(mail.outbox) - out, 1)


class NewslettersViewsTests(TestCase):
    def setUp(self):
        self.monthly = NewsletterList.objects.create(
            name="TestMonthly", slug="testmonthly"
        )
        self.segment_monthly = NewsletterSegment.objects.create(
            name="monthly fr", segment_id=3, newsletter_list=self.monthly, lang="fr"
        )
        self.n1 = Newsletter.objects.create(
            name="Newsletter1",
            newsletter_list=self.monthly,
            newsletter_segment=self.segment_monthly,
            published_at=datetime.now(),
            status=Newsletter.STATUS_DRAFT,
        )

        self.user = User.objects.create_user("adele", "adele@ulule.com", "$ecret")

    def test_newsletter_list(self):
        response = self.client.get(self.monthly.get_absolute_url())

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "courriers/newsletter_list.html")

    def test_newsletter_detail_view(self):
        response = self.client.get(self.n1.get_absolute_url())
        self.assertEqual(response.status_code, 404)

        self.n1.status = Newsletter.STATUS_ONLINE
        self.n1.save()

        response = self.client.get(self.n1.get_absolute_url())
        self.assertEqual(response.status_code, 200)

        self.assertTemplateUsed(response, "courriers/newsletter_detail.html")

    def test_newsletter_list_subscribe_view(self):
        response = self.client.get(
            reverse("newsletter_list_subscribe", kwargs={"slug": self.monthly.slug})
        )
        self.assertEqual(response.status_code, 200)

        response = self.client.get(
            reverse("newsletter_list_subscribe", kwargs={"slug": self.monthly.slug}),
            HTTP_X_REQUESTED_WITH="XMLHttpRequest",
        )
        self.assertEqual(response.status_code, 200)

        self.assertTrue(isinstance(response.context["form"], SubscriptionForm))

    def test_newsletter_list_unsubscribe_view(self):
        url = (
            reverse("newsletter_list_unsubscribe", kwargs={"slug": "testmonthly"})
            + "?email=adele@ulule.com"
        )

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "courriers/newsletter_list_unsubscribe.html")

        self.assertTrue(isinstance(response.context["form"], UnsubscribeForm))

        response = self.client.get(
            url,
            {"form": response.context["form"]},
            HTTP_X_REQUESTED_WITH="XMLHttpRequest",
        )
        self.assertEqual(response.status_code, 200)

    def test_newsletter_list_unsubscribe_complete(self):
        # Without email param
        valid_data = {"email": "adele@ulule.com"}

        NewsletterSubscriber.objects.create(
            newsletter_list=self.monthly, email="adele@ulule.com", user=self.user
        )

        response = self.client.post(
            reverse("newsletter_list_unsubscribe", kwargs={"slug": "testmonthly"}),
            data=valid_data,
        )

        self.assertRedirects(
            response,
            expected_url=reverse(
                "newsletter_list_unsubscribe_done", args=[self.monthly.slug]
            ),
            status_code=302,
            target_status_code=200,
        )

        subscriber = NewsletterSubscriber.objects.get(email="adele@ulule.com")

        self.assertFalse(subscriber.subscribed)

    def test_newsletter_list_all_unsubscribe_view(self):
        url = reverse("newsletter_list_unsubscribe") + "?email=adele@ulule.com"

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "courriers/newsletter_list_unsubscribe.html")

        self.assertTrue(isinstance(response.context["form"], UnsubscribeForm))

        response = self.client.get(
            url,
            {"form": response.context["form"]},
            HTTP_X_REQUESTED_WITH="XMLHttpRequest",
        )
        self.assertEqual(response.status_code, 200)

    def test_newsletter_list_all_unsubscribe_complete(self):
        url = reverse("newsletter_list_unsubscribe") + "?email=adele@ulule.com"
        NewsletterSubscriber.objects.create(
            newsletter_list=self.monthly, email="adele@ulule.com", user=self.user
        )

        # GET
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

        response = self.client.post(url, data={"email": "adele@ulule.com"})

        self.assertRedirects(
            response,
            expected_url=reverse("newsletter_list_unsubscribe_done"),
            status_code=302,
            target_status_code=200,
        )

        subscriber = NewsletterSubscriber.objects.get(email="adele@ulule.com")
        self.assertFalse(subscriber.subscribed)

    def test_newsletter_raw_detail_view(self):
        response = self.client.get(reverse("newsletter_raw_detail", kwargs={"pk": 1}))
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "courriers/newsletter_raw_detail.html")


class SubscribeFormTest(TestCase):
    def setUp(self):
        from courriers.backends import get_backend

        self.backend_klass = get_backend()
        self.backend = self.backend_klass()

        self.monthly = NewsletterList.objects.create(
            name="TestMonthly", slug="testmonthly"
        )
        self.segment_monthly = NewsletterSegment.objects.create(
            name="monthly fr", segment_id=3, newsletter_list=self.monthly, lang="fr"
        )

    def test_subscription_logged_in(self):
        self.client.login(username="thoas", password="secret")

        valid_data = {"receiver": "florent@ulule.com"}

        form = SubscriptionForm(data=valid_data, **{"newsletter_list": self.monthly})

        self.assertTrue(form.is_valid())

        user = User.objects.create(username="thoas", email="florent@ulule.com")
        form.save(user)

        new_subscriber = NewsletterSubscriber.objects.filter(
            email=valid_data["receiver"], user=user
        )

        self.assertEqual(new_subscriber.count(), 1)

        self.backend.unregister("florent@ulule.com")

    def test_subscribe_task(self):
        user = User.objects.create(username="adele-ulule", email="adele@ulule.com")
        subscribe.apply_async(
            kwargs={
                "email": "adele@ulule.com",
                "newsletter_list_id": self.monthly.pk,
                "lang": "fr",
            }
        )

        new_subscriber = NewsletterSubscriber.objects.filter(
            email="adele@ulule.com", is_unsubscribed=False
        )
        self.assertEqual(new_subscriber.count(), 1)


class NewDatetime(datetime.datetime):
    @classmethod
    def now(cls):
        return cls(2014, 4, 9, 9, 56, 2, 342715)


@mock.patch.object(settings, "BACKEND_CLASS", "courriers.backends.simple.SimpleBackend")
class UnsubscribeFormTest(TestCase):
    def setUp(self):
        from courriers.backends import get_backend

        datetime.datetime = NewDatetime

        self.backend_klass = get_backend()
        self.backend = self.backend_klass()

        self.monthly = NewsletterList.objects.create(
            name="TestMonthly", slug="testmonthly"
        )
        self.weekly = NewsletterList.objects.create(
            name="TestWeekly", slug="testweekly"
        )
        self.daily = NewsletterList.objects.create(name="TestDaily", slug="testdaily")

        self.user = User.objects.create_user("adele", "adele@ulule.com", "$ecret")

    def test_unsubscription(self):
        self.backend.unregister("adele@ulule.com", self.monthly, lang="fr")

        self.backend.register("adele@ulule.com", self.monthly, lang="fr")
        self.backend.register("adele@ulule.com", self.weekly, lang="fr")
        self.backend.register("adele@ulule.com", self.daily, lang="fr")

        # Unsubscribe from monthly
        valid_data = {"email": "adele@ulule.com", "from_all": False}

        form = UnsubscribeForm(
            data=valid_data,
            initial={"email": "adele@ulule.com"},
            **{"newsletter_list": self.monthly}
        )

        self.assertTrue(form.is_valid())

        form.save()

        old_subscriber = NewsletterSubscriber.objects.get(
            email=valid_data["email"], newsletter_list=self.monthly
        )
        old_subscriber2 = NewsletterSubscriber.objects.get(
            email=valid_data["email"], newsletter_list=self.weekly
        )

        self.assertEqual(old_subscriber.is_unsubscribed, True)
        self.assertEqual(old_subscriber.unsubscribed_at, datetime.now())
        self.assertEqual(old_subscriber2.is_unsubscribed, False)

        # Unsubscribe from all
        valid_data = {"email": "adele@ulule.com", "from_all": True}

        form = UnsubscribeForm(data=valid_data, newsletter_list=self.weekly)

        self.assertTrue(form.is_valid())

        form.save()

        old_subscriber = NewsletterSubscriber.objects.filter(
            email=valid_data["email"], newsletter_list=self.weekly
        )
        old_subscriber2 = NewsletterSubscriber.objects.filter(
            email=valid_data["email"], newsletter_list=self.daily
        )

        self.assertEqual(old_subscriber.get().is_unsubscribed, True)
        self.assertEqual(old_subscriber2.get().is_unsubscribed, True)
        self.assertEqual(old_subscriber2.get().unsubscribed_at, datetime.now())

    def test_unsubscription_logged_in(self):
        user = User.objects.create_user("thoas", "florent@ulule.com", "secret")

        self.backend.register("florent@ulule.com", self.monthly, "fr", user=user)
        new_subscriber = NewsletterSubscriber.objects.filter(
            email="florent@ulule.com", user=user
        )
        self.assertEqual(new_subscriber.count(), 1)

        valid_data = {"email": "florent@ulule.com", "from_all": False}

        form = UnsubscribeForm(
            data=valid_data,
            initial={"email": "florent@ulule.com"},
            **{"newsletter_list": self.monthly}
        )

        self.assertTrue(form.is_valid())

        form.save(user)  # Equivalent for saving with user logged in

        self.backend.unregister("florent@ulule.com")

    def test_unsubscribe_task(self):
        NewsletterSubscriber.objects.create(
            newsletter_list=self.monthly, email="adele@ulule.com", user=self.user
        )

        unsubscribe.apply_async(
            kwargs={"email": "adele@ulule.com", "newsletter_list_id": self.monthly.pk}
        )

        new_subscriber = NewsletterSubscriber.objects.filter(
            email="adele@ulule.com", newsletter_list=self.monthly, is_unsubscribed=True
        )
        self.assertEqual(new_subscriber.count(), 1)


if hasattr(settings, "COURRIERS_MAILJET_API_KEY") and hasattr(
    settings, "COURRIERS_MAILJET_API_SECRET_KEY"
):

    @mock.patch.object(
        settings, "BACKEND_CLASS", "courriers.backends.mailjet.MailjetBackend"
    )
    class SubscribeMailjetFormTest(SubscribeFormTest):
        pass

    @mock.patch.object(
        settings, "BACKEND_CLASS", "courriers.backends.mailjet.MailjetBackend"
    )
    class UnsubscribeMailjetFormTest(UnsubscribeFormTest):
        pass

    @mock.patch.object(
        settings, "BACKEND_CLASS", "courriers.backends.mailjet.MailjetBackend"
    )
    class MailjetBackendTests(BaseBackendTests):
        def test_registration(self):
            super(MailjetBackendTests, self).test_registration()

            for newsletter in self.newsletters:
                self.backend.send_mails(newsletter)


class NewsletterModelsTest(TestCase):
    def test_navigation(self):
        monthly = NewsletterList.objects.create(name="TestMonthly", slug="testmonthly")
        segment_monthly = NewsletterSegment.objects.create(
            name="monthly fr", segment_id=3, newsletter_list=monthly, lang="fr"
        )

        Newsletter.objects.create(
            name="Newsletter4",
            status=Newsletter.STATUS_DRAFT,
            published_at=datetime.now() - datetime.timedelta(hours=4),
            newsletter_list=monthly,
            newsletter_segment=segment_monthly,
        )
        n1 = Newsletter.objects.create(
            name="Newsletter1",
            status=Newsletter.STATUS_ONLINE,
            published_at=datetime.now() - datetime.timedelta(hours=3),
            newsletter_list=monthly,
            newsletter_segment=segment_monthly,
        )
        n2 = Newsletter.objects.create(
            name="Newsletter2",
            status=Newsletter.STATUS_ONLINE,
            published_at=datetime.now() - datetime.timedelta(hours=2),
            newsletter_list=monthly,
            newsletter_segment=segment_monthly,
        )
        n3 = Newsletter.objects.create(
            name="Newsletter3",
            status=Newsletter.STATUS_ONLINE,
            published_at=datetime.now() - datetime.timedelta(hours=1),
            newsletter_list=monthly,
            newsletter_segment=segment_monthly,
        )

        self.assertEqual(n2.get_previous(), n1)
        self.assertEqual(n2.get_next(), n3)
        self.assertEqual(n1.get_previous(), None)



class BaseBackend(object):
    def register(self, email, lang=None, user=None):
        raise NotImplemented

    def unregister(self, email, user=None):
        raise NotImplemented

    def exists(self, email, user=None):
        raise NotImplemented

    def send_mails(self, newsletter):
        raise NotImplemented



# -*- coding: utf-8 -*-
import six

from django.core import exceptions

try:
    from importlib import import_module
except ImportError:
    from django.utils.importlib import import_module


CLASS_PATH_ERROR = "django-courriers is unable to interpret settings value for %s. " "%s should be in the form of a tupple: " "('path.to.models.Class', 'app_label')."


def load_class(class_path, setting_name=None):
    """
    Loads a class given a class_path. The setting value may be a string or a
    tuple.

    The setting_name parameter is only there for pretty error output, and
    therefore is optional
    """
    if not isinstance(class_path, six.string_types):
        try:
            class_path, app_label = class_path
        except:
            if setting_name:
                raise exceptions.ImproperlyConfigured(
                    CLASS_PATH_ERROR % (setting_name, setting_name)
                )
            else:
                raise exceptions.ImproperlyConfigured(
                    CLASS_PATH_ERROR % ("this setting", "It")
                )

    try:
        class_module, class_name = class_path.rsplit(".", 1)
    except ValueError:
        if setting_name:
            txt = "%s isn't a valid module. Check your %s setting" % (
                class_path,
                setting_name,
            )
        else:
            txt = "%s isn't a valid module." % class_path
        raise exceptions.ImproperlyConfigured(txt)

    try:
        mod = import_module(class_module)
    except ImportError as e:
        if setting_name:
            txt = 'Error importing backend %s: "%s". Check your %s setting' % (
                class_module,
                e,
                setting_name,
            )
        else:
            txt = 'Error importing backend %s: "%s".' % (class_module, e)

        raise exceptions.ImproperlyConfigured(txt)

    try:
        clazz = getattr(mod, class_name)
    except AttributeError:
        if setting_name:
            txt = (
                'Backend module "%s" does not define a "%s" class. Check'
                " your %s setting" % (class_module, class_name, setting_name)
            )
        else:
            txt = 'Backend module "%s" does not define a "%s" class.' % (
                class_module,
                class_name,
            )
        raise exceptions.ImproperlyConfigured(txt)
    return clazz


def ajaxify_template_var(template_var):
    if isinstance(template_var, (list, tuple)):
        template_var = type(template_var)(
            ajaxify_template_name(name) for name in template_var
        )
    elif isinstance(template_var, basestring):
        template_var = ajaxify_template_name(template_var)
    return template_var


def ajaxify_template_name(name):
    if "." in name:
        name = "%s-ajax.%s" % tuple(name.rsplit(".", 1))
    else:
        name += "-ajax"
    return name



from __future__ import absolute_import

import os

from celery import Celery

from django.conf import settings

# set the default Django settings module for the 'celery' program.
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "courriers.tests.settings")

app = Celery("courriers")

# Using a string here means the worker will not have to
# pickle the object when using Windows.
app.config_from_object("django.conf:settings")
app.autodiscover_tasks(lambda: settings.INSTALLED_APPS)



from __future__ import absolute_import, unicode_literals

from mailjet_rest import Client

from django.core.exceptions import ImproperlyConfigured
from django.template.loader import render_to_string

from ..settings import (
    MAILJET_API_KEY,
    DEFAULT_FROM_EMAIL,
    DEFAULT_FROM_NAME,
    MAILJET_API_SECRET_KEY,
    PRE_PROCESSORS,
)
from .campaign import CampaignBackend
from ..utils import load_class


class MailjetRESTBackend(CampaignBackend):
    def __init__(self):
        if not MAILJET_API_KEY:
            raise ImproperlyConfigured(
                "Please specify your MAILJET API key in Django settings"
            )

        if not MAILJET_API_SECRET_KEY:
            raise ImproperlyConfigured(
                "Please specify your MAILJET API SECRET key in Django settings"
            )

        self.client = Client(auth=(MAILJET_API_KEY, MAILJET_API_SECRET_KEY))

    def _send_campaign(self, newsletter, list_id, segment_id=None):
        subject = newsletter.name

        options = {
            "Subject": subject,
            "ContactsListID": list_id,
            "Locale": "en",
            "SenderEmail": DEFAULT_FROM_EMAIL,
            "Sender": DEFAULT_FROM_NAME,
            "SenderName": DEFAULT_FROM_NAME,
            "Title": subject,
        }

        if segment_id:
            options["SegmentationID"] = segment_id

        context = {
            "object": newsletter,
            "items": newsletter.items.select_related("newsletter"),
            "options": options,
        }

        html = render_to_string("courriers/newsletter_raw_detail.html", context)
        text = render_to_string("courriers/newsletter_raw_detail.txt", context)

        for pre_processor in PRE_PROCESSORS:
            html = load_class(pre_processor)(html)

        res = self.client.campaigndraft.create(data=options)

        result = res.json()

        campaign_id = result["Data"][0]["ID"]

        data = {"Html-part": html, "Text-part": text}

        self.client.campaigndraft_detailcontent.create(id=campaign_id, data=data)
        self.client.campaigndraft_send.create(id=campaign_id)

    def subscribe(self, list_id, email, lang=None, user=None):
        data = {"Action": "addforce", "Contacts": [{"Email": email}]}

        self.client.contactslist_ManageManyContacts.create(id=list_id, data=data)

    def unsubscribe(self, list_id, email, lang=None, user=None):
        data = {"Action": "unsub", "Contacts": [{"Email": email}]}

        self.client.contactslist_ManageManyContacts.create(id=list_id, data=data)



import django.dispatch

unsubscribed = django.dispatch.Signal(
    providing_args=["user", "newsletter_list", "email"]
)

subscribed = django.dispatch.Signal(providing_args=["user", "newsletter_list", "email"])



import logging

from courriers.settings import FAIL_SILENTLY, DEFAULT_FROM_EMAIL, DEFAULT_FROM_NAME

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils import translation

from .simple import SimpleBackend

logger = logging.getLogger("courriers")


class CampaignBackend(SimpleBackend):
    def send_mails(self, newsletter):
        if not newsletter.is_online():
            raise Exception("This newsletter is not online. You can't send it.")

        nl_list = newsletter.newsletter_list
        nl_segment = newsletter.newsletter_segment
        self.send_campaign(newsletter, nl_list.list_id, nl_segment.segment_id)

    def _format_slug(self, *args):
        raise NotImplementedError

    def send_campaign(self, newsletter, list_id, segment_id):
        if not DEFAULT_FROM_EMAIL:
            raise ImproperlyConfigured(
                "You have to specify a DEFAULT_FROM_EMAIL in Django settings."
            )
        if not DEFAULT_FROM_NAME:
            raise ImproperlyConfigured(
                "You have to specify a DEFAULT_FROM_NAME in Django settings."
            )

        old_language = translation.get_language()
        language = newsletter.newsletter_segment.lang or settings.LANGUAGE_CODE

        translation.activate(language)

        try:
            self._send_campaign(newsletter, list_id, segment_id=segment_id)
        except Exception as e:
            logger.exception(e)

            if not FAIL_SILENTLY:
                raise e
        else:
            newsletter.sent = True
            newsletter.save(update_fields=("sent",))

        translation.activate(old_language)

    def subscribe(self, email, list_id):
        pass

    def unsubscribe(self, email, list_id):
        pass



from django.db import models
from django.utils import timezone as datetime
from django.utils.encoding import python_2_unicode_compatible
from django.contrib.auth.models import AbstractUser, UserManager
from django.dispatch import receiver

from courriers.compat import AUTH_USER_MODEL
from courriers.settings import ALLOWED_LANGUAGES
from courriers.models import NewsletterList
from courriers import signals


@receiver(signals.unsubscribed)
def handle_unsubscribed(user, newsletter_list=None, **kwargs):
    user.unsubscribe(newsletter_list.pk if newsletter_list else None)


@receiver(signals.subscribed)
def handle_subscribed(user, newsletter_list=None, **kwargs):
    user.subscribe(newsletter_list.pk if newsletter_list else None)


class User(AbstractUser):
    class Meta:
        abstract = False

    objects = UserManager()

    def subscribed(self, newsletter_list_id, lang=None):
        filters = {"user": self, "newsletter_list_id": newsletter_list_id}
        if lang:
            filters["lang"] = lang

        return NewsletterSubscriber.objects.filter(**filters).exists()

    def subscribe(self, newsletter_list_id, lang=None):
        if isinstance(newsletter_list_id, NewsletterList):
            newsletter_list_id = newsletter_list_id.pk

        if not self.subscribed(newsletter_list_id, lang=lang):
            NewsletterSubscriber.objects.create(
                subscribed_at=datetime.now(),
                user=self,
                email=self.email,
                lang=lang,
                newsletter_list_id=newsletter_list_id,
            )
        else:
            (
                NewsletterSubscriber.objects.filter(
                    user=self, newsletter_list_id=newsletter_list_id, lang=lang
                ).update(unsubscribed_at=None, is_unsubscribed=False)
            )

    def unsubscribe(self, newsletter_list_id=None, lang=None):
        if isinstance(newsletter_list_id, NewsletterList):
            newsletter_lists = [newsletter_list_id.pk]
        elif newsletter_list_id:
            newsletter_lists = NewsletterList.objects.filter(
                pk=newsletter_list_id
            ).values_list("pk", flat=True)
        else:
            newsletter_lists = NewsletterList.objects.all().values_list("pk", flat=True)

        qs = NewsletterSubscriber.objects.filter(
            user=self, email=self.email, newsletter_list_id__in=newsletter_lists
        )

        if lang:
            qs = qs.filter(lang=lang)

        qs.update(unsubscribed_at=datetime.now(), is_unsubscribed=True)


@python_2_unicode_compatible
class NewsletterSubscriber(models.Model):
    subscribed_at = models.DateTimeField(auto_now_add=True)
    user = models.ForeignKey(
        AUTH_USER_MODEL, on_delete=models.CASCADE, blank=True, null=True
    )
    is_unsubscribed = models.BooleanField(default=False, db_index=True)
    unsubscribed_at = models.DateTimeField(blank=True, null=True)
    email = models.EmailField(max_length=250)
    lang = models.CharField(
        max_length=10, blank=True, null=True, choices=ALLOWED_LANGUAGES
    )
    newsletter_list = models.ForeignKey(
        NewsletterList, related_name="newsletter_subscribers", on_delete=models.CASCADE
    )

    def __str__(self):
        return "%s for %s" % (self.email, self.newsletter_list)

    @property
    def subscribed(self):
        return self.is_unsubscribed is False



# -*- coding: utf-8 -*-
from .base import BaseBackend

from django.core import mail
from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from django.utils import translation
from django.contrib.auth import get_user_model

from ..settings import DEFAULT_FROM_EMAIL, PRE_PROCESSORS
from ..utils import load_class

User = get_user_model()


class SimpleBackend(BaseBackend):
    def subscribe(self, list_id, email, lang=None, user=None):
        pass

    def unsubscribe(self, list_id, email, lang=None, user=None):
        pass

    def register(self, email, newsletter_list, lang=None, user=None):
        if not user:
            try:
                user = User.objects.get(email=email)
            except User.DoesNotExist:
                user = User.objects.create(email=email, username=email)

        user.subscribe(newsletter_list, lang=lang)

    def unregister(self, email, newsletter_list=None, user=None, lang=None):
        if not user:
            try:
                user = User.objects.get(email=email)
            except User.DoesNotExist:
                return

        user.unsubscribe(newsletter_list, lang=lang)

    def send_mails(self, newsletter, fail_silently=False, subscribers=None):
        connection = mail.get_connection(fail_silently=fail_silently)

        emails = []

        old_language = translation.get_language()

        for subscriber in subscribers:
            if (
                newsletter.newsletter_segment.lang
                and newsletter.newsletter_segment.lang != subscriber.lang
            ):
                continue

            translation.activate(subscriber.lang)

            email = EmailMultiAlternatives(
                newsletter.name,
                render_to_string(
                    "courriers/newsletter_raw_detail.txt",
                    {"object": newsletter, "subscriber": subscriber},
                ),
                DEFAULT_FROM_EMAIL,
                [subscriber.email],
                connection=connection,
            )

            html = render_to_string(
                "courriers/newsletter_raw_detail.html",
                {
                    "object": newsletter,
                    "items": newsletter.items.all().prefetch_related("newsletter"),
                    "subscriber": subscriber,
                },
            )

            for pre_processor in PRE_PROCESSORS:
                html = load_class(pre_processor)(html)

            email.attach_alternative(html, "text/html")

            emails.append(email)

        translation.activate(old_language)

        results = connection.send_messages(emails)

        newsletter.sent = True
        newsletter.save(update_fields=("sent",))

        return results



from __future__ import unicode_literals

import os

from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey
from django.template.defaultfilters import slugify, truncatechars
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone as datetime
from django.urls import reverse
from django.utils.encoding import python_2_unicode_compatible
from django.db.models.query import QuerySet

from ..settings import ALLOWED_LANGUAGES


def get_file_path(instance, filename):
    fname, ext = os.path.splitext(filename)
    filename = "{}{}".format(slugify(truncatechars(fname, 50)), ext)

    return os.path.join("courriers", "uploads", filename)


@python_2_unicode_compatible
class NewsletterList(models.Model):
    name = models.CharField(max_length=255)
    slug = models.SlugField(max_length=255, unique=True)
    description = models.TextField(blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True)
    list_id = models.IntegerField(null=True)

    class Meta:
        abstract = True

    def __str__(self):
        return self.name or ""

    def get_absolute_url(self):
        return reverse("newsletter_list", kwargs={"slug": self.slug})


@python_2_unicode_compatible
class NewsletterSegment(models.Model):
    name = models.CharField(max_length=255)
    segment_id = models.IntegerField()
    newsletter_list = models.ForeignKey(
        NewsletterList, on_delete=models.PROTECT, related_name="lists"
    )
    lang = models.CharField(
        max_length=10, blank=True, null=True, choices=ALLOWED_LANGUAGES
    )

    class Meta:
        abstract = True

    def __str__(self):
        return self.name or ""


class NewsletterQuerySet(QuerySet):
    def status_online(self):
        return self.filter(
            status=Newsletter.STATUS_ONLINE, published_at__lt=datetime.now()
        ).order_by("published_at")

    def get_previous(self, current_date):
        return (
            self.status_online()
            .filter(published_at__lt=current_date)
            .order_by("-published_at")
            .first()
        )

    def get_next(self, current_date):
        return (
            self.status_online()
            .filter(published_at__gt=current_date)
            .order_by("-published_at")
            .first()
        )


class NewsletterManager(models.Manager):
    def get_queryset(self):
        return NewsletterQuerySet(self.model)

    def status_online(self):
        return self.get_queryset().status_online()

    def get_previous(self, current_date):
        return self.get_queryset().get_previous(current_date)

    def get_next(self, current_date):
        return self.get_queryset().get_next(current_date)


@python_2_unicode_compatible
class Newsletter(models.Model):
    STATUS_ONLINE = 1
    STATUS_DRAFT = 2

    STATUS_CHOICES = ((STATUS_ONLINE, _("Online")), (STATUS_DRAFT, _("Draft")))

    name = models.CharField(max_length=255)
    published_at = models.DateTimeField(null=True)
    status = models.PositiveIntegerField(
        choices=STATUS_CHOICES, default=STATUS_DRAFT, db_index=True
    )
    headline = models.TextField(blank=True, null=True)
    conclusion = models.TextField(blank=True, null=True)
    cover = models.ImageField(upload_to=get_file_path, blank=True, null=True)
    newsletter_list = models.ForeignKey(
        NewsletterList, related_name="newsletters", on_delete=models.PROTECT
    )
    newsletter_segment = models.ForeignKey(
        NewsletterSegment, related_name="segments", on_delete=models.PROTECT
    )
    sent = models.BooleanField(default=False, db_index=True)

    objects = NewsletterManager()

    class Meta:
        abstract = True

    def __str__(self):
        return self.name or ""

    def get_previous(self):
        return self.__class__.objects.filter(
            newsletter_list=self.newsletter_list_id
        ).get_previous(self.published_at)

    def get_next(self):
        return self.__class__.objects.filter(
            newsletter_list=self.newsletter_list_id
        ).get_next(self.published_at)

    def is_online(self):
        return self.status == self.STATUS_ONLINE

    def get_absolute_url(self):
        return reverse("newsletter_detail", args=[self.pk])


@python_2_unicode_compatible
class NewsletterItem(models.Model):
    newsletter = models.ForeignKey(
        Newsletter, related_name="items", on_delete=models.CASCADE
    )
    content_type = models.ForeignKey(
        ContentType, on_delete=models.PROTECT, blank=True, null=True
    )
    object_id = models.PositiveIntegerField(blank=True, null=True)
    content_object = GenericForeignKey("content_type", "object_id")
    name = models.CharField(max_length=255, blank=True, null=True)
    description = models.TextField(blank=True, null=True)
    image = models.ImageField(upload_to=get_file_path, blank=True, null=True)
    url = models.URLField(blank=True, null=True)
    position = models.PositiveIntegerField(null=True, blank=True)

    class Meta:
        ordering = ["position"]
        abstract = True

    def __str__(self):
        return self.name or ""



from django.conf.urls import url

from .views import (
    NewsletterListView,
    NewsletterDetailView,
    NewsletterListSubscribeView,
    NewsletterRawDetailView,
    NewsletterListUnsubscribeView,
    NewsletterListSubscribeDoneView,
    NewsletterListUnsubscribeDoneView,
)


urlpatterns = [
    url(
        r"^(?P<pk>(\d+))/detail/$",
        NewsletterDetailView.as_view(),
        name="newsletter_detail",
    ),
    url(
        r"^(?P<slug>(\w+))/subscribe/$",
        NewsletterListSubscribeView.as_view(),
        name="newsletter_list_subscribe",
    ),
    url(
        r"^(?P<pk>(\d+))/raw/$",
        NewsletterRawDetailView.as_view(),
        name="newsletter_raw_detail",
    ),
    url(
        r"^(?:(?P<slug>(\w+))/)?unsubscribe/$",
        NewsletterListUnsubscribeView.as_view(),
        name="newsletter_list_unsubscribe",
    ),
    url(
        r"^subscribe/done/$",
        NewsletterListSubscribeDoneView.as_view(),
        name="newsletter_list_subscribe_done",
    ),
    url(
        r"^unsubscribe/(?:(?P<slug>(\w+))/)?done/$",
        NewsletterListUnsubscribeDoneView.as_view(),
        name="newsletter_list_unsubscribe_done",
    ),
    url(
        r"^(?P<slug>(\w+))/(?:(?P<lang>(\w+))/)?(?:(?P<page>(\d+))/)?$",
        NewsletterListView.as_view(),
        name="newsletter_list",
    ),
]



from django.conf import settings


BACKEND_CLASS = getattr(
    settings, "COURRIERS_BACKEND_CLASS", "courriers.backends.simple.SimpleBackend"
)

MAILCHIMP_API_KEY = getattr(settings, "COURRIERS_MAILCHIMP_API_KEY", "")

MAILJET_API_KEY = getattr(settings, "COURRIERS_MAILJET_API_KEY", "")

MAILJET_CONTACTSLIST_LIMIT = getattr(
    settings, "COURRIERS_MAILJET_CONTACTSLIST_LIMIT", 1000
)

MAILJET_CONTACTFILTER_LIMIT = getattr(
    settings, "COURRIERS_MAILJET_CONTACTFILTER_LIMIT", 1000
)

MAILJET_API_SECRET_KEY = getattr(settings, "COURRIERS_MAILJET_API_SECRET_KEY", "")

DEFAULT_FROM_EMAIL = getattr(
    settings, "COURRIERS_DEFAULT_FROM_EMAIL", settings.DEFAULT_FROM_EMAIL
)

DEFAULT_FROM_NAME = getattr(settings, "COURRIERS_DEFAULT_FROM_NAME", "")

ALLOWED_LANGUAGES = getattr(settings, "COURRIERS_ALLOWED_LANGUAGES", settings.LANGUAGES)

PRE_PROCESSORS = getattr(settings, "COURRIERS_PRE_PROCESSORS", ())

PAGINATE_BY = getattr(settings, "COURRIERS_PAGINATE_BY", 9)

FAIL_SILENTLY = getattr(settings, "COURRIERS_FAIL_SILENTLY", False)

NEWSLETTERLIST_MODEL = getattr(
    settings,
    "COURRIERS_NEWSLETTERLIST_MODEL",
    "courriers.models.newsletterlist.NewsletterList",
)

NEWSLETTER_MODEL = getattr(
    settings, "COURRIERS_NEWSLETTER_MODEL", "courriers.models.newsletter.Newsletter"
)

NEWSLETTERITEM_MODEL = getattr(
    settings,
    "COURRIERS_NEWSLETTERITEM_MODEL",
    "courriers.models.newsletteritem.NewsletterItem",
)

NEWSLETTERSEGMENT_MODEL = getattr(
    settings,
    "COURRIERS_NEWSLETTERSEGMENT_MODEL",
    "courriers.models.newslettersegment.NewsletterSegment",
)



from courriers.utils import load_class

from courriers import settings


NewsletterList = load_class(settings.NEWSLETTERLIST_MODEL)

NewsletterSegment = load_class(settings.NEWSLETTERSEGMENT_MODEL)

Newsletter = load_class(settings.NEWSLETTER_MODEL)

NewsletterItem = load_class(settings.NEWSLETTERITEM_MODEL)
